package client

import (
	"context"
	"crypto/tls"
	"crypto/x509"
	"encoding/json"
	"errors"
	"gitlab.novgate.com/common/common"
	"gitlab.novgate.com/common/common/utils"
	"gitlab.novgate.com/common/system-client/pb"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials"
	"google.golang.org/grpc/credentials/insecure"
	"os"
	"time"
)

// NewGrpcClient 创建一个新的gRPC客户端
func NewGrpcClient(config *SystemConfig) (*GrpcClient, error) {
	var opts []grpc.DialOption
	if config.ProxyCA != "" {
		// 如果需要TLS连接，可以配置TLS选项
		// CA cert that signed the proxy
		f, err := os.ReadFile(config.ProxyCA)
		if err != nil {
			panic(err)
		}
		p := x509.NewCertPool()
		p.AppendCertsFromPEM(f)
		tlsConfig := &tls.Config{
			RootCAs: p,
		}
		opts = append(opts, grpc.WithTransportCredentials(credentials.NewTLS(tlsConfig)))
	} else {
		opts = append(opts, grpc.WithTransportCredentials(insecure.NewCredentials()))
	}

	conn, err := grpc.NewClient(config.ApiUrl, opts...)

	if err != nil {
		return nil, err
	}

	return &GrpcClient{
		appKey: config.AppKey,
		aes:    utils.NewAES([]byte(config.AppKey), []byte(config.AppSecret)),
		conn:   conn,
		client: pb.NewUserServiceClient(conn),
	}, nil
}

// Close 关闭gRPC连接
func (c *GrpcClient) Close() error {
	return c.conn.Close()
}

// SayHello 调用SayHello RPC方法
func (c *GrpcClient) Login(name, password string) (*pb.AdminUser, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.LoginReq{
		Header: &pb.RpcHeader{
			Uuid:  "33333333",
			Token: "",
		},
		UserName: name,
		Password: password,
	}

	rpcParam := c.encrypt(req)
	resp, err := c.client.Login(ctx, rpcParam)
	if err != nil {
		return nil, err
	}

	if resp.Code != common.Success {
		return nil, errors.New(resp.Msg)
	}

	respData := c.decrypt([]byte(resp.Data))
	var user pb.AdminUser
	if err := json.Unmarshal([]byte(respData), &user); err != nil {
		return nil, err
	}
	return &user, nil
}

func (c *GrpcClient) Logout(token string) error {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.RpcHeader{
		Uuid:  "33333333",
		Token: token,
	}
	rpcParam := c.encrypt(req)
	resp, err := c.client.Logout(ctx, rpcParam)
	if err != nil {
		return err
	}
	if resp.Code != common.Success {
		return errors.New(resp.Msg)
	}
	return nil
}

func (c *GrpcClient) CheckToken(token string) (*pb.AdminUser, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.RpcHeader{
		Uuid:  "33333333",
		Token: token,
	}
	rpcParam := c.encrypt(req)
	resp, err := c.client.CheckToken(ctx, rpcParam)
	if err != nil {
		return nil, err
	}
	if resp.Code != common.Success {
		return nil, errors.New(resp.Msg)
	}
	respData := c.decrypt([]byte(resp.Data))
	var user pb.AdminUser
	if err := json.Unmarshal([]byte(respData), &user); err != nil {
		return nil, err
	}
	return &user, nil
}

func (c *GrpcClient) GetUserMenu(token string) ([]*pb.MenuTreeModel, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.RpcHeader{
		Uuid:  "33333333",
		Token: token,
	}
	rpcParam := c.encrypt(req)
	resp, err := c.client.GetUserMenu(ctx, rpcParam)
	if err != nil {
		return nil, err
	}
	if resp.Code != common.Success {
		return nil, errors.New(resp.Msg)
	}
	respData := c.decrypt([]byte(resp.Data))
	var menu []*pb.MenuTreeModel
	if err := json.Unmarshal([]byte(respData), &menu); err != nil {
		return nil, err
	}
	return menu, nil
}
func (c *GrpcClient) GetRoleMenu(token string, roleId int64) ([]*pb.MenuTreeModel, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.UserRoleMenuReq{
		Header: &pb.RpcHeader{Token: token},
		RoleId: roleId,
	}

	rpcParam := c.encrypt(req)
	resp, err := c.client.GetUserRoleMenu(ctx, rpcParam)
	if err != nil {
		return nil, err
	}
	if resp.Code != common.Success {
		return nil, errors.New(resp.Msg)
	}
	respData := c.decrypt([]byte(resp.Data))
	var menu []*pb.MenuTreeModel
	if err := json.Unmarshal([]byte(respData), &menu); err != nil {
		return nil, err
	}
	return menu, nil
}

func (c *GrpcClient) UserList(name, loginName string, roleId int64) (*pb.UserListResp, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()

	req := &pb.UserListReq{
		Header: &pb.RpcHeader{
			Uuid:  "33333333",
			Token: "",
		},
		RoleId:    roleId,
		UserName:  name,
		LoginName: loginName,
		JobNum:    0,
	}

	rpcParam := c.encrypt(req)
	resp, err := c.client.UserList(ctx, rpcParam)
	if err != nil {
		return nil, err
	}

	if resp.Code != common.Success {
		return nil, errors.New(resp.Msg)
	}

	respData := c.decrypt([]byte(resp.Data))
	var user pb.UserListResp
	if err := json.Unmarshal([]byte(respData), &user); err != nil {
		return nil, err
	}
	return &user, nil
}
func (c *GrpcClient) decrypt(body []byte) string {
	decrypt, err := c.aes.Decrypt(body)
	if err != nil {
		return ""
	}

	return string(decrypt)
}

func (c *GrpcClient) encrypt(param interface{}) *pb.RpcParam {

	result := &pb.RpcParam{
		AppKey: c.appKey,
	}

	marshal, _ := json.Marshal(param)

	encrypt, err := c.aes.Encrypt(marshal)
	if err != nil {
		return nil
	}
	result.Param = encrypt
	return result
}
